@@ -8,9 +8,7 @@ from trytond.pyson import Eval, Bool
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 
-from trytond.modules.product import price_digits
-
-__all__ = ['ShipmentIn', 'Move']
+from trytond.modules.product import price_digits, round_price
 
 
 class ShipmentIn(metaclass=PoolMeta):
@@ -19,61 +17,98 @@ class ShipmentIn(metaclass=PoolMeta):
             'readonly': Eval('state') != 'draft',
             },
         depends=['state'])
-    cost_currency = fields.Many2One('currency.currency', 'Cost Currency',
+
+    cost_currency_used = fields.Function(fields.Many2One(
+            'currency.currency', "Cost Currency",
+            states={
+                'invisible': Eval('cost_edit', False),
+                },
+            depends=['cost_edit']),
+        'on_change_with_cost_currency_used')
+    cost_currency = fields.Many2One(
+        'currency.currency', "Cost Currency",
         states={
             'required': Bool(Eval('cost')),
-            'readonly': ~Eval('state').in_(['draft', 'assigned', 'packed']),
-            }, depends=['cost', 'state'])
-    cost_currency_digits = fields.Function(fields.Integer(
-            'Cost Currency Digits'),
-        'on_change_with_cost_currency_digits')
-    cost = fields.Numeric('Cost', digits=(16, Eval('cost_currency_digits', 2)),
+            'invisible': ~Eval('cost_edit', False),
+            'readonly': ~Eval('state').in_(['draft']),
+            },
+        depends=['cost', 'cost_edit', 'state'])
+    cost_used = fields.Function(fields.Numeric(
+            "Cost", digits=price_digits,
+            states={
+                'invisible': Eval('cost_edit', False),
+                },
+            depends=['cost_edit']),
+        'on_change_with_cost_used')
+    cost = fields.Numeric(
+        "Cost", digits=price_digits,
         states={
-            'readonly': ~Eval('state').in_(['draft', 'assigned', 'packed']),
-            }, depends=['carrier', 'state', 'cost_currency_digits'])
+            'invisible': ~Eval('cost_edit', False),
+            'readonly': ~Eval('state').in_(['draft']),
+            },
+        depends=['cost_edit', 'state'])
 
-    @fields.depends('cost_currency')
-    def on_change_with_cost_currency_digits(self, name=None):
-        if self.cost_currency:
-            return self.cost_currency.digits
-        return 2
+    cost_edit = fields.Boolean(
+        "Edit Cost",
+        states={
+            'readonly': ~Eval('state').in_(['draft']),
+            },
+        depends=['state'],
+        help="Check to edit the cost.")
 
     def _get_carrier_context(self):
         return {}
 
-    @fields.depends(methods=['on_change_incoming_moves'])
-    def on_change_carrier(self):
-        self.on_change_incoming_moves()
+    @fields.depends('carrier', methods=['_get_carrier_context'])
+    def _compute_costs(self):
+        costs = {
+            'cost': None,
+            'cost_currency': None,
+            }
+        if self.carrier:
+            with Transaction().set_context(self._get_carrier_context()):
+                cost, currency_id = self.carrier.get_purchase_price()
+            costs['cost'] = round_price(cost)
+            costs['cost_currency'] = currency_id
+        return costs
 
-    @fields.depends('carrier', 'incoming_moves')
-    def on_change_incoming_moves(self):
-        try:
-            super(ShipmentIn, self).on_change_incoming_moves()
-        except AttributeError:
-            pass
-        if not self.carrier:
-            return
-        with Transaction().set_context(self._get_carrier_context()):
-            cost, currency_id = self.carrier.get_purchase_price()
-        self.cost = cost
-        self.cost_currency = currency_id
-        if self.cost_currency:
-            self.cost_currency_digits = self.cost_currency.digits
+    @fields.depends(
+        'state', 'cost_currency', 'cost_edit', methods=['_compute_costs'])
+    def on_change_with_cost_currency_used(self, name=None):
+        if (not self.cost_edit
+                and self.state not in {'cancelled', 'received', 'done'}):
+            return self._compute_costs()['cost_currency']
+        elif self.cost_currency:
+            return self.cost_currency.id
+
+    @fields.depends('state', 'cost', 'cost_edit', methods=['_compute_costs'])
+    def on_change_with_cost_used(self, name=None):
+        if (not self.cost_edit
+                and self.state not in {'cancelled', 'received', 'done'}):
+            return self._compute_costs()['cost']
         else:
-            self.cost_currency_digits = 2
+            return self.cost
+
+    @fields.depends(
+        'cost_edit', 'cost_used', 'cost_currency_used')
+    def on_change_cost_edit(self):
+        if self.cost_edit:
+            self.cost = self.cost_used
+            self.cost_currency = self.cost_currency_used
 
     def allocate_cost_by_value(self):
         pool = Pool()
         Currency = pool.get('currency.currency')
         Move = pool.get('stock.move')
 
-        if not self.cost:
+        if not self.cost_used:
             return
 
-        cost = Currency.compute(self.cost_currency, self.cost,
-            self.company.currency, round=False)
+        cost = Currency.compute(
+            self.cost_currency_used, self.cost_used, self.company.currency,
+            round=False)
         moves = [m for m in self.incoming_moves
-            if m.state not in ('done', 'cancel')]
+            if m.state not in ('done', 'cancelled')]
 
         sum_value = 0
         unit_prices = {}
@@ -93,8 +128,8 @@ class ShipmentIn(metaclass=PoolMeta):
                 move_cost = cost / Decimal(len(moves))
             else:
                 move_cost = cost * quantity * unit_prices[move.id] / sum_value
-            unit_shipment_cost = (move_cost / quantity).quantize(exp,
-                rounding=ROUND_DOWN)
+            unit_shipment_cost = round_price(
+                move_cost / quantity, rounding=ROUND_DOWN)
             costs.append({
                     'unit_shipment_cost': unit_shipment_cost,
                     'difference': move_cost - (unit_shipment_cost * quantity),
@@ -116,8 +151,8 @@ class ShipmentIn(metaclass=PoolMeta):
             unit_shipment_cost = Currency.compute(
                 self.company.currency, cost['unit_shipment_cost'],
                 move.currency, round=False)
-            unit_shipment_cost = unit_shipment_cost.quantize(
-                exp, rounding=ROUND_HALF_EVEN)
+            unit_shipment_cost = round_price(
+                unit_shipment_cost, rounding=ROUND_HALF_EVEN)
             move.unit_price += unit_shipment_cost
             move.unit_shipment_cost = unit_shipment_cost
         Move.save(moves)
@@ -128,6 +163,8 @@ class ShipmentIn(metaclass=PoolMeta):
     def receive(cls, shipments):
         Carrier = Pool().get('carrier')
         for shipment in shipments:
+            shipment.cost = shipment.cost_used
+            shipment.cost_currency = shipment.cost_currency_used
             if shipment.carrier:
                 allocation_method = \
                     shipment.carrier.carrier_cost_allocation_method
@@ -137,13 +174,39 @@ class ShipmentIn(metaclass=PoolMeta):
             getattr(shipment, 'allocate_cost_by_%s' % allocation_method)()
         super(ShipmentIn, cls).receive(shipments)
 
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('cancelled')
+    def cancel(cls, shipments):
+        for shipment in shipments:
+            shipment.cost = None
+            shipment.cost_currency = None
+        cls.save(shipments)
+        super().cancel(shipments)
+
 
 class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
     unit_shipment_cost = fields.Numeric('Unit Shipment Cost',
         digits=price_digits, readonly=True)
 
-    # Split the shipment cost if account_stock_continental is installed
+    def _compute_unit_price(self, unit_price):
+        if self.unit_shipment_cost:
+            unit_price -= self.unit_shipment_cost
+        unit_price = super()._compute_unit_price(unit_price)
+        if self.unit_shipment_cost:
+            unit_price += self.unit_shipment_cost
+        return unit_price
+
+    def _compute_component_unit_price(self, unit_price):
+        if self.unit_shipment_cost:
+            unit_price -= self.unit_shipment_cost
+        unit_price = super()._compute_component_unit_price(unit_price)
+        if self.unit_shipment_cost:
+            unit_price += self.unit_shipment_cost
+        return unit_price
+
+    # Split the shipment cost if account_stock_continental is activated
     def _get_account_stock_move_lines(self, type_):
         pool = Pool()
         AccountMoveLine = pool.get('account.move.line')
@@ -158,7 +221,7 @@ class Move(metaclass=PoolMeta):
                 self.company.currency)
             shipment_cost_account = \
                 self.shipment.carrier.carrier_product.account_expense_used
-            account = self.product.account_stock_supplier_used
+            account = self.product.account_stock_in_used
             for move_line in move_lines:
                 if move_line.account == account:
                     move_line.credit -= shipment_cost
@@ -173,16 +236,19 @@ class Move(metaclass=PoolMeta):
                 raise AssertionError('missing account_stock_supplier')
         return move_lines
 
-    # Remove shipment cost if account_stock_anglo_saxon is installed
+    # Remove shipment cost if account_stock_anglo_saxon is activated
     @classmethod
     def _get_anglo_saxon_move(cls, moves, quantity, type_):
         pool = Pool()
         Currency = pool.get('currency.currency')
+        Uom = pool.get('product.uom')
         for move, qty, cost_price in super(Move, cls)._get_anglo_saxon_move(
                 moves, quantity, type_):
             if (type_.startswith('in_')
                     and move.unit_shipment_cost):
+                shipment_cost = Uom.compute_price(move.uom,
+                    move.unit_shipment_cost, move.product.default_uom)
                 shipment_cost = Currency.compute(move.currency,
-                    move.unit_shipment_cost, move.company.currency)
+                    shipment_cost, move.company.currency)
                 cost_price -= shipment_cost
             yield move, qty, cost_price
